@@ -3,7 +3,7 @@
  * com.apple.mobile.lockdownd service implementation.
  *
  * Copyright (c) 2009-2015 Martin Szulecki All Rights Reserved.
- * Copyright (c) 2014 Nikias Bassen. All Rights Reserved.
+ * Copyright (c) 2014-2015 Nikias Bassen All Rights Reserved.
  * Copyright (c) 2010 Bryan Forbes All Rights Reserved.
  * Copyright (c) 2008 Zach C. All Rights Reserved.
  *
@@ -33,15 +33,6 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <unistd.h>
-#ifdef HAVE_OPENSSL
-#include <openssl/pem.h>
-#include <openssl/x509.h>
-#include <openssl/x509v3.h>
-#else
-#include <libtasn1.h>
-#include <gnutls/x509.h>
-#include <gnutls/crypto.h>
-#endif
 #include <plist/plist.h>
 
 #include "property_list_service.h"
@@ -57,6 +48,46 @@
 #define sleep(x) Sleep(x*1000)
 #endif
 
+struct st_lockdownd_error_str_map {
+	const char *lockdown_errstr;
+	const char *errstr;
+	lockdownd_error_t errcode;
+};
+
+static struct st_lockdownd_error_str_map lockdownd_error_str_map[] = {
+	{ "InvalidResponse", "Invalid response", LOCKDOWN_E_INVALID_RESPONSE },
+	{ "MissingKey", "Missing key", LOCKDOWN_E_MISSING_KEY },
+	{ "MissingValue", "Missing value", LOCKDOWN_E_MISSING_VALUE },
+	{ "GetProhibited", "Get value prohibited", LOCKDOWN_E_GET_PROHIBITED },
+	{ "SetProhibited", "Set value prohibited", LOCKDOWN_E_SET_PROHIBITED },
+	{ "RemoveProhibited", "Remove value prohibited", LOCKDOWN_E_REMOVE_PROHIBITED },
+	{ "ImmutableValue", "Immutable value", LOCKDOWN_E_IMMUTABLE_VALUE },
+	{ "PasswordProtected", "Password protected", LOCKDOWN_E_PASSWORD_PROTECTED },
+	{ "UserDeniedPairing", "User denied pairing", LOCKDOWN_E_USER_DENIED_PAIRING },
+	{ "PairingDialogResponsePending", "Pairing dialog response pending", LOCKDOWN_E_PAIRING_DIALOG_RESPONSE_PENDING },
+	{ "MissingHostID", "Missing HostID", LOCKDOWN_E_MISSING_HOST_ID },
+	{ "InvalidHostID", "Invalid HostID", LOCKDOWN_E_INVALID_HOST_ID },
+	{ "SessionActive", "Session active", LOCKDOWN_E_SESSION_ACTIVE },
+	{ "SessionInactive", "Session inactive", LOCKDOWN_E_SESSION_INACTIVE },
+	{ "MissingSessionID", "Missing session ID", LOCKDOWN_E_MISSING_SESSION_ID },
+	{ "InvalidSessionID", "Invalid session ID", LOCKDOWN_E_INVALID_SESSION_ID },
+	{ "MissingService", "Missing service", LOCKDOWN_E_MISSING_SERVICE },
+	{ "InvalidService", "Invalid service", LOCKDOWN_E_INVALID_SERVICE },
+	{ "ServiceLimit", "Service limit reached", LOCKDOWN_E_SERVICE_LIMIT },
+	{ "MissingPairRecord", "Missing pair record", LOCKDOWN_E_MISSING_PAIR_RECORD },
+	{ "SavePairRecordFailed", "Saving pair record failed", LOCKDOWN_E_SAVE_PAIR_RECORD_FAILED },
+	{ "InvalidPairRecord", "Invalid pair record", LOCKDOWN_E_INVALID_PAIR_RECORD },
+	{ "InvalidActivationRecord", "Invalid activation record", LOCKDOWN_E_INVALID_ACTIVATION_RECORD },
+	{ "MissingActivationRecord", "Missing activation record", LOCKDOWN_E_MISSING_ACTIVATION_RECORD },
+	{ "ServiceProhibited", "Service prohibited", LOCKDOWN_E_SERVICE_PROHIBITED },
+	{ "EscrowLocked", "Escrow lockded", LOCKDOWN_E_ESCROW_LOCKED },
+	{ "PairingProhibitedOverThisConnection", "Pairing prohibited over this connection", LOCKDOWN_E_PAIRING_PROHIBITED_OVER_THIS_CONNECTION },
+	{ "FMiPProtected", "Find My iPhone/iPod/iPad protected", LOCKDOWN_E_FMIP_PROTECTED },
+	{ "MCProtected", "MC protected" , LOCKDOWN_E_MC_PROTECTED },
+	{ "MCChallengeRequired", "MC challenge required", LOCKDOWN_E_MC_CHALLENGE_REQUIRED },
+	{ NULL, NULL, 0 }
+};
+
 /**
  * Convert an error string identifier to a lockdownd_error_t value.
  * Used internally to get correct error codes from a response.
@@ -69,64 +100,46 @@
 static lockdownd_error_t lockdownd_strtoerr(const char* name)
 {
 	lockdownd_error_t err = LOCKDOWN_E_UNKNOWN_ERROR;
-
-	if (strcmp(name, "InvalidResponse") == 0) {
-		err = LOCKDOWN_E_INVALID_RESPONSE;
-	} else if (strcmp(name, "MissingKey") == 0) {
-		err = LOCKDOWN_E_MISSING_KEY;
-	} else if (strcmp(name, "MissingValue") == 0) {
-		err = LOCKDOWN_E_MISSING_VALUE;
-	} else if (strcmp(name, "GetProhibited") == 0) {
-		err = LOCKDOWN_E_GET_PROHIBITED;
-	} else if (strcmp(name, "SetProhibited") == 0) {
-		err = LOCKDOWN_E_SET_PROHIBITED;
-	} else if (strcmp(name, "RemoveProhibited") == 0) {
-		err = LOCKDOWN_E_REMOVE_PROHIBITED;
-	} else if (strcmp(name, "ImmutableValue") == 0) {
-		err = LOCKDOWN_E_IMMUTABLE_VALUE;
-	} else if (strcmp(name, "PasswordProtected") == 0) {
-		err = LOCKDOWN_E_PASSWORD_PROTECTED;
-	} else if (strcmp(name, "UserDeniedPairing") == 0) {
-		err = LOCKDOWN_E_USER_DENIED_PAIRING;
-	} else if (strcmp(name, "PairingDialogResponsePending") == 0) {
-		err = LOCKDOWN_E_PAIRING_DIALOG_RESPONSE_PENDING;
-	} else if (strcmp(name, "MissingHostID") == 0) {
-		err = LOCKDOWN_E_MISSING_HOST_ID;
-	} else if (strcmp(name, "InvalidHostID") == 0) {
-		err = LOCKDOWN_E_INVALID_HOST_ID;
-	} else if (strcmp(name, "SessionActive") == 0) {
-		err = LOCKDOWN_E_SESSION_ACTIVE;
-	} else if (strcmp(name, "SessionInactive") == 0) {
-		err = LOCKDOWN_E_SESSION_INACTIVE;
-	} else if (strcmp(name, "MissingSessionID") == 0) {
-		err = LOCKDOWN_E_MISSING_SESSION_ID;
-	} else if (strcmp(name, "InvalidSessionID") == 0) {
-		err = LOCKDOWN_E_INVALID_SESSION_ID;
-	} else if (strcmp(name, "MissingService") == 0) {
-		err = LOCKDOWN_E_MISSING_SERVICE;
-	} else if (strcmp(name, "InvalidService") == 0) {
-		err = LOCKDOWN_E_INVALID_SERVICE;
-	} else if (strcmp(name, "ServiceLimit") == 0) {
-		err = LOCKDOWN_E_SERVICE_LIMIT;
-	} else if (strcmp(name, "MissingPairRecord") == 0) {
-		err = LOCKDOWN_E_MISSING_PAIR_RECORD;
-	} else if (strcmp(name, "SavePairRecordFailed") == 0) {
-		err = LOCKDOWN_E_SAVE_PAIR_RECORD_FAILED;
-	} else if (strcmp(name, "InvalidPairRecord") == 0) {
-		err = LOCKDOWN_E_INVALID_PAIR_RECORD;
-	} else if (strcmp(name, "InvalidActivationRecord") == 0) {
-		err = LOCKDOWN_E_INVALID_ACTIVATION_RECORD;
-	} else if (strcmp(name, "MissingActivationRecord") == 0) {
-		err = LOCKDOWN_E_MISSING_ACTIVATION_RECORD;
-	} else if (strcmp(name, "ServiceProhibited") == 0) {
-		err = LOCKDOWN_E_SERVICE_PROHIBITED;
-	} else if (strcmp(name, "EscrowLocked") == 0) {
-		err = LOCKDOWN_E_ESCROW_LOCKED;
+	int i = 0;
+	while (lockdownd_error_str_map[i].lockdown_errstr) {
+		if (strcmp(lockdownd_error_str_map[i].lockdown_errstr, name) == 0) {
+			return lockdownd_error_str_map[i].errcode;
+		}
+		i++;
 	}
-
 	return err;
 }
 
+/**
+ * Convert a property_list_service_error_t value to a lockdownd_error_t
+ * value. Used internally to get correct error codes.
+ *
+ * @param err A property_list_service_error_t error code
+ *
+ * @return A matching lockdownd_error_t error code,
+ *     LOCKDOWND_E_UNKNOWN_ERROR otherwise.
+ */
+static lockdownd_error_t lockdownd_error(property_list_service_error_t err)
+{
+	switch (err) {
+		case PROPERTY_LIST_SERVICE_E_SUCCESS:
+			return LOCKDOWN_E_SUCCESS;
+		case PROPERTY_LIST_SERVICE_E_INVALID_ARG:
+			return LOCKDOWN_E_INVALID_ARG;
+		case PROPERTY_LIST_SERVICE_E_PLIST_ERROR:
+			return LOCKDOWN_E_PLIST_ERROR;
+		case PROPERTY_LIST_SERVICE_E_MUX_ERROR:
+			return LOCKDOWN_E_MUX_ERROR;
+		case PROPERTY_LIST_SERVICE_E_SSL_ERROR:
+			return LOCKDOWN_E_SSL_ERROR;
+		case PROPERTY_LIST_SERVICE_E_RECEIVE_TIMEOUT:
+			return LOCKDOWN_E_RECEIVE_TIMEOUT;
+		default:
+			break;
+	}
+	return LOCKDOWN_E_UNKNOWN_ERROR;
+}
+
 /**
  * Internally used function for checking the result from lockdown's answer
  * plist to a previously sent request.
@@ -181,7 +194,7 @@ static lockdownd_error_t lockdown_check_result(plist_t dict, const char *query_m
 					ret = lockdownd_strtoerr(err_value);
 					free(err_value);
 				} else {
-					debug_info("ERROR: unknown error occured");
+					debug_info("ERROR: unknown error occurred");
 				}
 			}
 			return ret;
@@ -341,18 +354,8 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_receive(lockdownd_client_t clie
 {
 	if (!client || !plist || (plist && *plist))
 		return LOCKDOWN_E_INVALID_ARG;
-	lockdownd_error_t ret = LOCKDOWN_E_SUCCESS;
-	property_list_service_error_t err;
 
-	err = property_list_service_receive_plist(client->parent, plist);
-	if (err != PROPERTY_LIST_SERVICE_E_SUCCESS) {
-		ret = LOCKDOWN_E_UNKNOWN_ERROR;
-	}
-
-	if (!*plist)
-		ret = LOCKDOWN_E_PLIST_ERROR;
-
-	return ret;
+	return lockdownd_error(property_list_service_receive_plist(client->parent, plist));
 }
 
 LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_send(lockdownd_client_t client, plist_t plist)
@@ -360,14 +363,7 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_send(lockdownd_client_t client,
 	if (!client || !plist)
 		return LOCKDOWN_E_INVALID_ARG;
 
-	lockdownd_error_t ret = LOCKDOWN_E_SUCCESS;
-	property_list_service_error_t err;
-
-	err = property_list_service_send_xml_plist(client->parent, plist);
-	if (err != PROPERTY_LIST_SERVICE_E_SUCCESS) {
-		ret = LOCKDOWN_E_UNKNOWN_ERROR;
-	}
-	return ret;
+	return lockdownd_error(property_list_service_send_xml_plist(client->parent, plist));
 }
 
 LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_query_type(lockdownd_client_t client, char **type)
@@ -629,7 +625,7 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_get_device_name(lockdownd_clien
 
 LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_client_new(idevice_t device, lockdownd_client_t *client, const char *label)
 {
-	if (!client)
+	if (!device || !client)
 		return LOCKDOWN_E_INVALID_ARG;
 
 	static struct lockdownd_service_descriptor service = {
@@ -647,6 +643,7 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_client_new(idevice_t device, lo
 	client_loc->parent = plistclient;
 	client_loc->ssl_enabled = 0;
 	client_loc->session_id = NULL;
+	client_loc->mux_id = device->mux_id;
 
 	if (idevice_get_udid(device, &client_loc->udid) != IDEVICE_E_SUCCESS) {
 		debug_info("failed to get device udid.");
@@ -678,21 +675,33 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_client_new_with_handshake(idevi
 	}
 
 	/* perform handshake */
-	if (LOCKDOWN_E_SUCCESS != lockdownd_query_type(client_loc, &type)) {
+	ret = lockdownd_query_type(client_loc, &type);
+	if (LOCKDOWN_E_SUCCESS != ret) {
 		debug_info("QueryType failed in the lockdownd client.");
-		ret = LOCKDOWN_E_NOT_ENOUGH_DATA;
-	} else {
-		if (strcmp("com.apple.mobile.lockdown", type)) {
-			debug_info("Warning QueryType request returned \"%s\".", type);
-		}
+	} else if (strcmp("com.apple.mobile.lockdown", type)) {
+		debug_info("Warning QueryType request returned \"%s\".", type);
 	}
 	free(type);
 
+	if (device->version == 0) {
+		plist_t p_version = NULL;
+		if (lockdownd_get_value(client_loc, NULL, "ProductVersion", &p_version) == LOCKDOWN_E_SUCCESS) {
+			int vers[3] = {0, 0, 0};
+			char *s_version = NULL;
+			plist_get_string_val(p_version, &s_version);
+			if (s_version && sscanf(s_version, "%d.%d.%d", &vers[0], &vers[1], &vers[2]) >= 2) {
+				device->version = DEVICE_VERSION(vers[0], vers[1], vers[2]);
+			}
+			free(s_version);
+		}
+		plist_free(p_version);
+	}
+
 	userpref_read_pair_record(client_loc->udid, &pair_record);
 	if (pair_record) {
 		pair_record_get_host_id(pair_record, &host_id);
 	}
-	if (LOCKDOWN_E_SUCCESS == ret && !host_id) {
+	if (LOCKDOWN_E_SUCCESS == ret && pair_record && !host_id) {
 		ret = LOCKDOWN_E_INVALID_CONF;
 	}
 
@@ -704,18 +713,18 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_client_new_with_handshake(idevi
 	plist_free(pair_record);
 	pair_record = NULL;
 
-	/* in any case, we need to validate pairing to receive trusted host status */
-	ret = lockdownd_validate_pair(client_loc, NULL);
+	if (device->version < DEVICE_VERSION(7,0,0)) {
+		/* for older devices, we need to validate pairing to receive trusted host status */
+		ret = lockdownd_validate_pair(client_loc, NULL);
 
-	/* if not paired yet, let's do it now */
-	if (LOCKDOWN_E_INVALID_HOST_ID == ret) {
-		free(host_id);
-		host_id = NULL;
-		ret = lockdownd_pair(client_loc, NULL);
-		if (LOCKDOWN_E_SUCCESS == ret) {
-			ret = lockdownd_validate_pair(client_loc, NULL);
-		} else if (LOCKDOWN_E_PAIRING_DIALOG_RESPONSE_PENDING == ret) {
-			debug_info("Device shows the pairing dialog.");
+		/* if not paired yet, let's do it now */
+		if (LOCKDOWN_E_INVALID_HOST_ID == ret) {
+			free(host_id);
+			host_id = NULL;
+			ret = lockdownd_pair(client_loc, NULL);
+			if (LOCKDOWN_E_SUCCESS == ret) {
+				ret = lockdownd_validate_pair(client_loc, NULL);
+			}
 		}
 	}
 
@@ -840,6 +849,8 @@ leave:
  *    the pair records from the current machine are used. New records will be
  *    generated automatically when pairing is done for the first time.
  * @param verb This is either "Pair", "ValidatePair" or "Unpair".
+ * @param options The pairing options to pass.
+ * @param response If non-NULL a pointer to lockdownd's response dictionary is returned.
  *
  * @return LOCKDOWN_E_SUCCESS on success, NP_E_INVALID_ARG when client is NULL,
  *  LOCKDOWN_E_PLIST_ERROR if the pair_record certificates are wrong,
@@ -847,7 +858,7 @@ leave:
  *  LOCKDOWN_E_PASSWORD_PROTECTED if the device is password protected,
  *  LOCKDOWN_E_INVALID_HOST_ID if the device does not know the caller's host id
  */
-static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_pair_record_t pair_record, const char *verb)
+static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_pair_record_t pair_record, const char *verb, plist_t options, plist_t *result)
 {
 	if (!client)
 		return LOCKDOWN_E_INVALID_ARG;
@@ -883,12 +894,8 @@ static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_
 			lockdownd_get_value(client, NULL, "WiFiAddress", &wifi_node);
 		} else {
 			/* use existing pair record */
-			if (userpref_has_pair_record(client->udid)) {
-				userpref_read_pair_record(client->udid, &pair_record_plist);
-				if (!pair_record_plist) {
-					return LOCKDOWN_E_INVALID_CONF;
-				}
-			} else {
+			userpref_read_pair_record(client->udid, &pair_record_plist);
+			if (!pair_record_plist) {
 				return LOCKDOWN_E_INVALID_HOST_ID;
 			}
 		}
@@ -907,9 +914,9 @@ static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_
 	plist_dict_set_item(dict, "Request", plist_new_string(verb));
 	plist_dict_set_item(dict, "ProtocolVersion", plist_new_string(LOCKDOWN_PROTOCOL_VERSION));
 
-	plist_t options = plist_new_dict();
-	plist_dict_set_item(options, "ExtendedPairingErrors", plist_new_bool(1));
-	plist_dict_set_item(dict, "PairingOptions", options);
+	if (options) {
+		plist_dict_set_item(dict, "PairingOptions", plist_copy(options));
+	}
 
 	/* send to device */
 	ret = lockdownd_send(client, dict);
@@ -960,8 +967,6 @@ static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_
 					if (extra_node && plist_get_node_type(extra_node) == PLIST_DATA) {
 						debug_info("Saving EscrowBag from response in pair record");
 						plist_dict_set_item(pair_record_plist, USERPREF_ESCROW_BAG_KEY, plist_copy(extra_node));
-						plist_free(extra_node);
-						extra_node = NULL;
 					}
 
 					/* save previously retrieved wifi mac address in pair record */
@@ -972,7 +977,7 @@ static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_
 						wifi_node = NULL;
 					}
 
-					userpref_save_pair_record(client->udid, pair_record_plist);
+					userpref_save_pair_record(client->udid, client->mux_id, pair_record_plist);
 				}
 			}
 		} else {
@@ -991,9 +996,6 @@ static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_
 				ret = lockdownd_strtoerr(value);
 				free(value);
 			}
-
-			plist_free(error_node);
-			error_node = NULL;
 		}
 	}
 
@@ -1007,25 +1009,42 @@ static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_
 		wifi_node = NULL;
 	}
 
-	plist_free(dict);
-	dict = NULL;
+	if (result) {
+		*result = dict;
+	} else {
+		plist_free(dict);
+		dict = NULL;
+	}
 
 	return ret;
 }
 
 LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_pair(lockdownd_client_t client, lockdownd_pair_record_t pair_record)
 {
-	return lockdownd_do_pair(client, pair_record, "Pair");
+
+	plist_t options = plist_new_dict();
+	plist_dict_set_item(options, "ExtendedPairingErrors", plist_new_bool(1));
+
+	lockdownd_error_t ret = lockdownd_do_pair(client, pair_record, "Pair", options, NULL);
+
+	plist_free(options);
+
+	return ret;
+}
+
+LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_pair_with_options(lockdownd_client_t client, lockdownd_pair_record_t pair_record, plist_t options, plist_t *response)
+{
+	return lockdownd_do_pair(client, pair_record, "Pair", options, response);
 }
 
 LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_validate_pair(lockdownd_client_t client, lockdownd_pair_record_t pair_record)
 {
-	return lockdownd_do_pair(client, pair_record, "ValidatePair");
+	return lockdownd_do_pair(client, pair_record, "ValidatePair", NULL, NULL);
 }
 
 LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_unpair(lockdownd_client_t client, lockdownd_pair_record_t pair_record)
 {
-	return lockdownd_do_pair(client, pair_record, "Unpair");
+	return lockdownd_do_pair(client, pair_record, "Unpair", NULL, NULL);
 }
 
 LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_enter_recovery(lockdownd_client_t client)
@@ -1095,7 +1114,6 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_goodbye(lockdownd_client_t clie
 LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_start_session(lockdownd_client_t client, const char *host_id, char **session_id, int *ssl_enabled)
 {
 	lockdownd_error_t ret = LOCKDOWN_E_SUCCESS;
-	property_list_service_error_t plret;
 	plist_t dict = NULL;
 
 	if (!client || !host_id)
@@ -1166,20 +1184,14 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_start_session(lockdownd_client_
 			debug_info("Failed to get SessionID!");
 		}
 
-		debug_info("Enable SSL Session: %s", (use_ssl?"true":"false"));
+		debug_info("Enable SSL Session: %s", (use_ssl ? "true" : "false"));
 
 		if (use_ssl) {
-			plret = property_list_service_enable_ssl(client->parent);
-			if (plret == PROPERTY_LIST_SERVICE_E_SUCCESS) {
-				ret = LOCKDOWN_E_SUCCESS;
-				client->ssl_enabled = 1;
-			} else {
-				ret = LOCKDOWN_E_SSL_ERROR;
-				client->ssl_enabled = 0;
-			}
+			ret = lockdownd_error(property_list_service_enable_ssl(client->parent));
+			client->ssl_enabled = (ret == LOCKDOWN_E_SUCCESS ? 1 : 0);
 		} else {
-			client->ssl_enabled = 0;
 			ret = LOCKDOWN_E_SUCCESS;
+			client->ssl_enabled = 0;
 		}
 	}
 
@@ -1504,3 +1516,41 @@ LIBIMOBILEDEVICE_API lockdownd_error_t lockdownd_service_descriptor_free(lockdow
 
 	return LOCKDOWN_E_SUCCESS;
 }
+
+LIBIMOBILEDEVICE_API const char* lockdownd_strerror(lockdownd_error_t err)
+{
+	switch (err) {
+		case LOCKDOWN_E_SUCCESS:
+			return "Success";
+		case LOCKDOWN_E_INVALID_ARG:
+			return "Invalid argument";
+		case LOCKDOWN_E_INVALID_CONF:
+			return "Invalid configuration";
+		case LOCKDOWN_E_PLIST_ERROR:
+			return "PropertyList error";
+		case LOCKDOWN_E_PAIRING_FAILED:
+			return "Pairing failed";
+		case LOCKDOWN_E_SSL_ERROR:
+			return "SSL error";
+		case LOCKDOWN_E_DICT_ERROR:
+			return "Invalid dictionary";
+		case LOCKDOWN_E_RECEIVE_TIMEOUT:
+			return "Receive timeout";
+		case LOCKDOWN_E_MUX_ERROR:
+			return "Mux error";
+		case LOCKDOWN_E_NO_RUNNING_SESSION:
+			return "No running session";
+		case LOCKDOWN_E_UNKNOWN_ERROR:
+			return "Unknown Error";
+		default: {
+			int i = 0;
+			while (lockdownd_error_str_map[i].lockdown_errstr) {
+				if (lockdownd_error_str_map[i].errcode == err) {
+					return lockdownd_error_str_map[i].errstr;
+				}
+				i++;
+			}
+		} break;
+	}
+	return "Unknown Error";
+}
